#pragma once

#ifndef __REDRATCONNECTION_H__
#define __REDRATCONNECTION_H__

#include "resource.h"       // main symbols
#include "SerialPort.h"
#include "RedRat.h"
#include "RedRatCP.h"

/////////////////////////////////////////////////////////////////////////////
// CRedRatConnection
class ATL_NO_VTABLE CRedRatConnection : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CRedRatConnection, &CLSID_RedRatConnection>,
	public ISupportErrorInfoImpl<&IID_IRedRatConnection>,
	public IConnectionPointContainerImpl<CRedRatConnection>,
	public IDispatchImpl<IRedRatConnection, &IID_IRedRatConnection, &LIBID_REDRATLib>,
  public IProvideClassInfo2Impl<&CLSID_RedRatConnection, NULL, &LIBID_REDRATLib>,
  public CProxy_IRedRatConnectionEvents<CRedRatConnection>
{
public:
//Constructors / Destructors
	CRedRatConnection();
	virtual ~CRedRatConnection();

DECLARE_PROTECT_FINAL_CONSTRUCT()
DECLARE_NO_REGISTRY()

BEGIN_CONNECTION_POINT_MAP(CRedRatConnection)
CONNECTION_POINT_ENTRY(DIID__IRedRatConnectionEvents)
END_CONNECTION_POINT_MAP()

BEGIN_COM_MAP(CRedRatConnection)
	COM_INTERFACE_ENTRY(IRedRatConnection)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
  COM_INTERFACE_ENTRY(IProvideClassInfo)
  COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

//Non COM related functions
  HRESULT Open(long nPort);
  void Close();
  static UINT _MonitorThread(LPVOID pVoid);
  UINT MonitorThread();
  void HandleDataFromPort(CString& sSerialString);
  BOOL ReadResponse(CString& sResponse, BOOL& bError, CString& sErrorDescription);
  void TerminateMonitoringThread();
  HRESULT StartMonitoringThread();
  HRESULT _StartCaptureMode();

//IRedRatConnection
  STDMETHOD(get_Timeout)(long* pVal);
  STDMETHOD(put_Timeout)(long Val);
  STDMETHOD(get_Port)(long* pVal);
  STDMETHOD(get_FirmwareDetails)(BSTR* pVal);
  STDMETHOD(Send)(BSTR Signal);
  STDMETHOD(StartCaptureMode)();
  STDMETHOD(TerminateCaptureMode)();
  STDMETHOD(Blink)();
  STDMETHOD(ReadFromEEPROM)(long Location, long* pVal);
  STDMETHOD(WriteToEEPROM)(long Location, long Value);
  STDMETHOD(get_AutoReceiveSignals)(VARIANT_BOOL* pVal);
  STDMETHOD(put_AutoReceiveSignals)(VARIANT_BOOL Val);

//Member variables
  long                    m_nPort;
  CSerialPort             m_Port;
  CComAutoCriticalSection m_csPort;
  CEvent                  m_StopEvent;
  CEvent                  m_PortEvent;
  CWinThread*             m_pMonitorThread;
  CComAutoCriticalSection m_csThread;
  OVERLAPPED              m_Overlapped;
  long                    m_lTimeout;
  LONG volatile           m_bAutoReceiveSignals;
};

#endif //__REDRATCONNECTION_H__
